@@ -4,28 +4,25 @@
 
     Transforms for LaTeX builder.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2021 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from typing import cast
+from typing import Any, Dict, List, Set, Tuple, cast
 
 from docutils import nodes
+from docutils.nodes import Element, Node
+from docutils.transforms.references import Substitutions
 
 from sphinx import addnodes
-from sphinx.builders.latex.nodes import (
-    captioned_literal_block, footnotemark, footnotetext, math_reference, thebibliography
-)
+from sphinx.application import Sphinx
+from sphinx.builders.latex.nodes import (captioned_literal_block, footnotemark, footnotetext,
+                                         math_reference, thebibliography)
 from sphinx.domains.citation import CitationDomain
 from sphinx.transforms import SphinxTransform
 from sphinx.transforms.post_transforms import SphinxPostTransform
 from sphinx.util.nodes import NodeMatcher
 
-if False:
-    # For type annotation
-    from typing import Any, Dict, List, Set, Tuple  # NOQA
-    from sphinx.application import Sphinx  # NOQA
-
 URI_SCHEMES = ('mailto:', 'http:', 'https:', 'ftp:')
 
 
@@ -34,13 +31,24 @@ class FootnoteDocnameUpdater(SphinxTransform):
     default_priority = 700
     TARGET_NODES = (nodes.footnote, nodes.footnote_reference)
 
-    def apply(self, **kwargs):
-        # type: (Any) -> None
+    def apply(self, **kwargs: Any) -> None:
         matcher = NodeMatcher(*self.TARGET_NODES)
         for node in self.document.traverse(matcher):  # type: nodes.Element
             node['docname'] = self.env.docname
 
 
+class SubstitutionDefinitionsRemover(SphinxPostTransform):
+    """Remove ``substitution_definition node from doctrees."""
+
+    # should be invoked after Substitutions process
+    default_priority = Substitutions.default_priority + 1
+    builders = ('latex',)
+
+    def run(self, **kwargs: Any) -> None:
+        for node in self.document.traverse(nodes.substitution_definition):
+            node.parent.remove(node)
+
+
 class ShowUrlsTransform(SphinxPostTransform):
     """Expand references to inline text or footnotes.
 
@@ -54,8 +62,7 @@ class ShowUrlsTransform(SphinxPostTransform):
     # references are expanded to footnotes (or not)
     expanded = False
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         try:
             # replace id_prefix temporarily
             settings = self.document.settings  # type: Any
@@ -69,8 +76,7 @@ class ShowUrlsTransform(SphinxPostTransform):
             # restore id_prefix
             settings.id_prefix = id_prefix
 
-    def expand_show_urls(self):
-        # type: () -> None
+    def expand_show_urls(self) -> None:
         show_urls = self.config.latex_show_urls
         if show_urls is False or show_urls == 'no':
             return
@@ -93,8 +99,7 @@ class ShowUrlsTransform(SphinxPostTransform):
                         textnode = nodes.Text(" (%s)" % uri)
                         node.parent.insert(index + 1, textnode)
 
-    def get_docname_for_node(self, node):
-        # type: (nodes.Node) -> str
+    def get_docname_for_node(self, node: Node) -> str:
         while node:
             if isinstance(node, nodes.document):
                 return self.env.path2doc(node['source'])
@@ -105,8 +110,7 @@ class ShowUrlsTransform(SphinxPostTransform):
 
         return None  # never reached here. only for type hinting
 
-    def create_footnote(self, uri, docname):
-        # type: (str, str) -> Tuple[nodes.footnote, nodes.footnote_reference]
+    def create_footnote(self, uri: str, docname: str) -> Tuple[nodes.footnote, nodes.footnote_reference]:  # NOQA
         reference = nodes.reference('', nodes.Text(uri), refuri=uri, nolinkurl=True)
         footnote = nodes.footnote(uri, auto=1, docname=docname)
         footnote['names'].append('#')
@@ -122,8 +126,7 @@ class ShowUrlsTransform(SphinxPostTransform):
 
         return footnote, footnote_ref
 
-    def renumber_footnotes(self):
-        # type: () -> None
+    def renumber_footnotes(self) -> None:
         collector = FootnoteCollector(self.document)
         self.document.walkabout(collector)
 
@@ -153,31 +156,26 @@ class ShowUrlsTransform(SphinxPostTransform):
 class FootnoteCollector(nodes.NodeVisitor):
     """Collect footnotes and footnote references on the document"""
 
-    def __init__(self, document):
-        # type: (nodes.document) -> None
+    def __init__(self, document: nodes.document) -> None:
         self.auto_footnotes = []            # type: List[nodes.footnote]
         self.used_footnote_numbers = set()  # type: Set[str]
         self.footnote_refs = []             # type: List[nodes.footnote_reference]
         super().__init__(document)
 
-    def unknown_visit(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_visit(self, node: Node) -> None:
         pass
 
-    def unknown_departure(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_departure(self, node: Node) -> None:
         pass
 
-    def visit_footnote(self, node):
-        # type: (nodes.footnote) -> None
+    def visit_footnote(self, node: nodes.footnote) -> None:
         if node.get('auto'):
             self.auto_footnotes.append(node)
         else:
             for name in node['names']:
                 self.used_footnote_numbers.add(name)
 
-    def visit_footnote_reference(self, node):
-        # type: (nodes.footnote_reference) -> None
+    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None:
         self.footnote_refs.append(node)
 
 
@@ -194,7 +192,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
                   headings having footnotes
                   <footnote_reference>
                       1
-              <footnote ids="1">
+              <footnote ids="id1">
                   <label>
                       1
                   <paragraph>
@@ -205,11 +203,9 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
           <section>
               <title>
                   headings having footnotes
-                  <footnotemark>
+                  <footnotemark refid="id1">
                       1
-              <footnotetext>
-                  footnote body
-              <footnotetext>
+              <footnotetext ids="id1">
                   <label>
                       1
                   <paragraph>
@@ -224,7 +220,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
               1
           blah blah blah ...
 
-          <footnote ids="1">
+          <footnote ids="id1">
               <label>
                   1
               <paragraph>
@@ -233,7 +229,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
       After::
 
           blah blah blah
-          <footnote ids="1">
+          <footnote ids="id1">
               <label>
                   1
               <paragraph>
@@ -253,7 +249,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
               1
           blah blah blah ...
 
-          <footnote ids="1">
+          <footnote ids="id1">
               <label>
                   1
               <paragraph>
@@ -262,13 +258,13 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
       After::
 
           blah blah blah
-          <footnote ids="1">
+          <footnote ids="id1">
               <label>
                   1
               <paragraph>
                   footnote body
           blah blah blah
-          <footnotemark>
+          <footnotemark refid="id1">
               1
           blah blah blah ...
 
@@ -276,7 +272,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
 
       Before::
 
-          <footnote ids="1">
+          <footnote ids="id1">
               <label>
                   1
               <paragraph>
@@ -293,26 +289,26 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
           <table>
               <title>
                   title having footnote_reference
-                  <footnote_reference refid="1">
+                  <footnote_reference refid="id1">
                       1
               <tgroup>
                   <thead>
                       <row>
                           <entry>
                               header having footnote_reference
-                              <footnote_reference refid="2">
+                              <footnote_reference refid="id2">
                                   2
                   <tbody>
                       <row>
                       ...
 
-          <footnote ids="1">
+          <footnote ids="id1">
               <label>
                   1
               <paragraph>
                   footnote body
 
-          <footnote ids="2">
+          <footnote ids="id2">
               <label>
                   2
               <paragraph>
@@ -323,23 +319,23 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
           <table>
               <title>
                   title having footnote_reference
-                  <footnotemark>
+                  <footnotemark refid="id1">
                       1
               <tgroup>
                   <thead>
                       <row>
                           <entry>
                               header having footnote_reference
-                              <footnotemark>
+                              <footnotemark refid="id2">
                                   2
                   <tbody>
-                      <footnotetext>
+                      <footnotetext ids="id1">
                           <label>
                               1
                           <paragraph>
                               footnote body
 
-                      <footnotetext>
+                      <footnotetext ids="id2">
                           <label>
                               2
                           <paragraph>
@@ -351,8 +347,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
     default_priority = 600
     builders = ('latex',)
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         footnotes = list(self.document.traverse(nodes.footnote))
         for node in footnotes:
             node.parent.remove(node)
@@ -362,8 +357,7 @@ class LaTeXFootnoteTransform(SphinxPostTransform):
 
 
 class LaTeXFootnoteVisitor(nodes.NodeVisitor):
-    def __init__(self, document, footnotes):
-        # type: (nodes.document, List[nodes.footnote]) -> None
+    def __init__(self, document: nodes.document, footnotes: List[nodes.footnote]) -> None:
         self.appeared = set()       # type: Set[Tuple[str, str]]
         self.footnotes = footnotes  # type: List[nodes.footnote]
         self.pendings = []          # type: List[nodes.footnote]
@@ -371,60 +365,48 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
         self.restricted = None      # type: nodes.Element
         super().__init__(document)
 
-    def unknown_visit(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_visit(self, node: Node) -> None:
         pass
 
-    def unknown_departure(self, node):
-        # type: (nodes.Node) -> None
+    def unknown_departure(self, node: Node) -> None:
         pass
 
-    def restrict(self, node):
-        # type: (nodes.Element) -> None
+    def restrict(self, node: Element) -> None:
         if self.restricted is None:
             self.restricted = node
 
-    def unrestrict(self, node):
-        # type: (nodes.Element) -> None
+    def unrestrict(self, node: Element) -> None:
         if self.restricted == node:
             self.restricted = None
             pos = node.parent.index(node)
             for i, footnote, in enumerate(self.pendings):
-                fntext = footnotetext('', *footnote.children)
+                fntext = footnotetext('', *footnote.children, ids=footnote['ids'])
                 node.parent.insert(pos + i + 1, fntext)
             self.pendings = []
 
-    def visit_figure(self, node):
-        # type: (nodes.figure) -> None
+    def visit_figure(self, node: nodes.figure) -> None:
         self.restrict(node)
 
-    def depart_figure(self, node):
-        # type: (nodes.figure) -> None
+    def depart_figure(self, node: nodes.figure) -> None:
         self.unrestrict(node)
 
-    def visit_term(self, node):
-        # type: (nodes.term) -> None
+    def visit_term(self, node: nodes.term) -> None:
         self.restrict(node)
 
-    def depart_term(self, node):
-        # type: (nodes.term) -> None
+    def depart_term(self, node: nodes.term) -> None:
         self.unrestrict(node)
 
-    def visit_caption(self, node):
-        # type: (nodes.caption) -> None
+    def visit_caption(self, node: nodes.caption) -> None:
         self.restrict(node)
 
-    def depart_caption(self, node):
-        # type: (nodes.caption) -> None
+    def depart_caption(self, node: nodes.caption) -> None:
         self.unrestrict(node)
 
-    def visit_title(self, node):
-        # type: (nodes.title) -> None
+    def visit_title(self, node: nodes.title) -> None:
         if isinstance(node.parent, (nodes.section, nodes.table)):
             self.restrict(node)
 
-    def depart_title(self, node):
-        # type: (nodes.title) -> None
+    def depart_title(self, node: nodes.title) -> None:
         if isinstance(node.parent, nodes.section):
             self.unrestrict(node)
         elif isinstance(node.parent, nodes.table):
@@ -432,45 +414,39 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
             self.pendings = []
             self.unrestrict(node)
 
-    def visit_thead(self, node):
-        # type: (nodes.thead) -> None
+    def visit_thead(self, node: nodes.thead) -> None:
         self.restrict(node)
 
-    def depart_thead(self, node):
-        # type: (nodes.thead) -> None
+    def depart_thead(self, node: nodes.thead) -> None:
         self.table_footnotes += self.pendings
         self.pendings = []
         self.unrestrict(node)
 
-    def depart_table(self, node):
-        # type: (nodes.table) -> None
+    def depart_table(self, node: nodes.table) -> None:
         tbody = list(node.traverse(nodes.tbody))[0]
         for footnote in reversed(self.table_footnotes):
-            fntext = footnotetext('', *footnote.children)
+            fntext = footnotetext('', *footnote.children, ids=footnote['ids'])
             tbody.insert(0, fntext)
 
         self.table_footnotes = []
 
-    def visit_footnote(self, node):
-        # type: (nodes.footnote) -> None
+    def visit_footnote(self, node: nodes.footnote) -> None:
         self.restrict(node)
 
-    def depart_footnote(self, node):
-        # type: (nodes.footnote) -> None
+    def depart_footnote(self, node: nodes.footnote) -> None:
         self.unrestrict(node)
 
-    def visit_footnote_reference(self, node):
-        # type: (nodes.footnote_reference) -> None
+    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None:
         number = node.astext().strip()
         docname = node['docname']
         if self.restricted:
-            mark = footnotemark('', number)
+            mark = footnotemark('', number, refid=node['refid'])
             node.replace_self(mark)
             if (docname, number) not in self.appeared:
                 footnote = self.get_footnote_by_reference(node)
                 self.pendings.append(footnote)
         elif (docname, number) in self.appeared:
-            mark = footnotemark('', number)
+            mark = footnotemark('', number, refid=node['refid'])
             node.replace_self(mark)
         else:
             footnote = self.get_footnote_by_reference(node)
@@ -481,8 +457,7 @@ class LaTeXFootnoteVisitor(nodes.NodeVisitor):
         self.appeared.add((docname, number))
         raise nodes.SkipNode
 
-    def get_footnote_by_reference(self, node):
-        # type: (nodes.footnote_reference) -> nodes.footnote
+    def get_footnote_by_reference(self, node: nodes.footnote_reference) -> nodes.footnote:
         docname = node['docname']
         for footnote in self.footnotes:
             if docname == footnote['docname'] and footnote['ids'][0] == node['refid']:
@@ -524,8 +499,7 @@ class BibliographyTransform(SphinxPostTransform):
     default_priority = 750
     builders = ('latex',)
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         citations = thebibliography()
         for node in self.document.traverse(nodes.citation):
             node.parent.remove(node)
@@ -544,8 +518,7 @@ class CitationReferenceTransform(SphinxPostTransform):
     default_priority = 5  # before ReferencesResolver
     builders = ('latex',)
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         domain = cast(CitationDomain, self.env.get_domain('citation'))
         matcher = NodeMatcher(addnodes.pending_xref, refdomain='citation', reftype='ref')
         for node in self.document.traverse(matcher):  # type: addnodes.pending_xref
@@ -565,8 +538,7 @@ class MathReferenceTransform(SphinxPostTransform):
     default_priority = 5  # before ReferencesResolver
     builders = ('latex',)
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         equations = self.env.get_domain('math').data['objects']
         for node in self.document.traverse(addnodes.pending_xref):
             if node['refdomain'] == 'math' and node['reftype'] in ('eq', 'numref'):
@@ -581,8 +553,7 @@ class LiteralBlockTransform(SphinxPostTransform):
     default_priority = 400
     builders = ('latex',)
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         matcher = NodeMatcher(nodes.container, literal_block=True)
         for node in self.document.traverse(matcher):  # type: nodes.container
             newnode = captioned_literal_block('', *node.children, **node.attributes)
@@ -594,15 +565,14 @@ class DocumentTargetTransform(SphinxPostTransform):
     default_priority = 400
     builders = ('latex',)
 
-    def run(self, **kwargs):
-        # type: (Any) -> None
+    def run(self, **kwargs: Any) -> None:
         for node in self.document.traverse(addnodes.start_of_file):
             section = node.next_node(nodes.section)
             if section:
                 section['ids'].append(':doc')  # special label for :doc:
 
 
-class IndexInSectionTitleTransform(SphinxTransform):
+class IndexInSectionTitleTransform(SphinxPostTransform):
     """Move index nodes in section title to outside of the title.
 
     LaTeX index macro is not compatible with some handling of section titles
@@ -629,8 +599,9 @@ class IndexInSectionTitleTransform(SphinxTransform):
             ...
     """
     default_priority = 400
+    builders = ('latex',)
 
-    def apply(self):
+    def run(self, **kwargs: Any) -> None:
         for node in self.document.traverse(nodes.title):
             if isinstance(node.parent, nodes.section):
                 for i, index in enumerate(node.traverse(addnodes.index)):
@@ -639,9 +610,9 @@ class IndexInSectionTitleTransform(SphinxTransform):
                     node.parent.insert(i + 1, index)
 
 
-def setup(app):
-    # type: (Sphinx) -> Dict[str, Any]
+def setup(app: Sphinx) -> Dict[str, Any]:
     app.add_transform(FootnoteDocnameUpdater)
+    app.add_post_transform(SubstitutionDefinitionsRemover)
     app.add_post_transform(BibliographyTransform)
     app.add_post_transform(CitationReferenceTransform)
     app.add_post_transform(DocumentTargetTransform)
